// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2010 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
package de.uka.ilkd.key.rtsj.rule.metaconstruct;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.expression.literal.IntLiteral;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Namespace;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.AbstractMetaOperator;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

public class WorstCaseArraySize extends AbstractMetaOperator {
    
    public WorstCaseArraySize() {
        super(new Name("#wcArraySize"), 2);
    }
    
    public Term calculate(Term term, SVInstantiations svInst, Services services) {
        TermFactory tf = TermFactory.DEFAULT;
        Namespace funcs = services.getNamespaces().functions();
        Function add = (Function) funcs.lookup(new Name("add"));
        Function mul = (Function) funcs.lookup(new Name("mul"));
        
        IntLiteral lit = new IntLiteral("16");
        Term sixteen =  services.getTypeConverter().convertToLogicElement(lit);
        lit = new IntLiteral("18");
        Term eightteen =  services.getTypeConverter().convertToLogicElement(lit);
        lit = new IntLiteral("19");
        Term nineteen =  services.getTypeConverter().convertToLogicElement(lit);
        
        if(convertToDecimalString(term.sub(0), services).equals("8") ||
           convertToDecimalString(term.sub(0), services).equals("4")){
            return tf.createFunctionTerm(add, sixteen, tf.createFunctionTerm(
                    mul, term.sub(0), term.sub(1)));
        }else if(convertToDecimalString(term.sub(0), services).equals("2")){
            return tf.createFunctionTerm(add, eightteen, tf.createFunctionTerm(
                    mul, term.sub(0), term.sub(1)));            
        }else{
            return tf.createFunctionTerm(add, nineteen, tf.createFunctionTerm(
                    mul, term.sub(0), term.sub(1)));            
        }
    }
}
