// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2010 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.rule.updatesimplifier;

import de.uka.ilkd.key.collection.ImmutableArray;
import de.uka.ilkd.key.collection.DefaultImmutableSet;
import de.uka.ilkd.key.collection.ImmutableSet;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.Location;
import de.uka.ilkd.key.logic.op.LocationVariable;
import de.uka.ilkd.key.logic.op.NonRigidFunctionLocation;
import de.uka.ilkd.key.logic.op.QuantifiableVariable;
import de.uka.ilkd.key.rule.AbstractUpdateRule;
import de.uka.ilkd.key.rule.UpdateSimplifier;

/**
 * Rule used by the update simplifier as application of an update on
 * a local variable or static variable
 * Implements rule (S1) described in ....
 */
public class ApplyOnLocalVariableOrStaticField extends AbstractUpdateRule {

    /**
     * creates an instance of this rule used by the given update
     * simplifier
     */
    public ApplyOnLocalVariableOrStaticField(UpdateSimplifier us) {
	super(us);
    }

    /**
     * determines whether this rule is applicable for the pair of
     * update and target (the term on which the update will be
     * applied) term. 
     * @param update the Update to be applied on target
     * @param target the Term on which the update is applied
     * @return true if the top level operator of target is a local
     * variable
     */
    public boolean isApplicable(Update update, Term target) {	
	return target.op() instanceof LocationVariable
               || target.op() instanceof NonRigidFunctionLocation 
                     && target.op().arity() == 0;	    
    }

    /**     
     */
    public Term apply(Update update, Term target, Services services) {
        logEnter(update, target);
	
        final Term result =
            UpdateSimplifierTermFactory.DEFAULT
            .createIfExCascade ( createCascade ( update, target ), target );
        
	logExit(result);

	return result;
    }
    
    private PVIfExCascade createCascade (Update update, Term target) {
        return new PVIfExCascade ( update.getAssignmentPairs
                                   ( (Location)target.op () ) );
    }

    private static class PVIfExCascade extends IterateAssignmentPairsIfExCascade {
        
        public PVIfExCascade (ImmutableArray<AssignmentPair> pairs) {
            super ( pairs );
        }
        
        public Term getCondition () {
            return getCurrentPair ().guard ();
        }
        
        protected ImmutableSet<QuantifiableVariable> criticalVars () {
            return DefaultImmutableSet.<QuantifiableVariable>nil();
        }
    }
    
    public Term matchingCondition (Update update, Term target, Services services) {
        return UpdateSimplifierTermFactory.DEFAULT
            .matchingCondition ( createCascade ( update, target ) );
    }
}
